# -*- coding: utf-8 -*-
#
# Copyright (C) 2013 Branko Majic
#
# This file is part of Django Conntrackt.
#
# Django Conntrackt is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# Django Conntrackt is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# Django Conntrackt.  If not, see <http://www.gnu.org/licenses/>.
#


# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Project'
        db.create_table('conntrackt_project', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=100)),
            ('description', self.gf('django.db.models.fields.TextField')(blank=True)),
        ))
        db.send_create_signal('conntrackt', ['Project'])

        # Adding model 'Location'
        db.create_table('conntrackt_location', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=100)),
            ('description', self.gf('django.db.models.fields.TextField')(blank=True)),
        ))
        db.send_create_signal('conntrackt', ['Location'])

        # Adding model 'Entity'
        db.create_table('conntrackt_entity', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=100)),
            ('description', self.gf('django.db.models.fields.TextField')(blank=True)),
            ('project', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['conntrackt.Project'])),
            ('location', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['conntrackt.Location'])),
        ))
        db.send_create_signal('conntrackt', ['Entity'])

        # Adding model 'Interface'
        db.create_table('conntrackt_interface', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(default='eth0', max_length=100)),
            ('description', self.gf('django.db.models.fields.TextField')(default='Main network interface.', blank=True)),
            ('entity', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['conntrackt.Entity'])),
            ('address', self.gf('django.db.models.fields.IPAddressField')(max_length=15)),
            ('netmask', self.gf('django.db.models.fields.IPAddressField')(default='255.255.255.255', max_length=15)),
        ))
        db.send_create_signal('conntrackt', ['Interface'])

        # Adding model 'Communication'
        db.create_table('conntrackt_communication', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('source', self.gf('django.db.models.fields.related.ForeignKey')(related_name='source_set', to=orm['conntrackt.Interface'])),
            ('destination', self.gf('django.db.models.fields.related.ForeignKey')(related_name='destination_set', to=orm['conntrackt.Interface'])),
            ('protocol', self.gf('django.db.models.fields.CharField')(max_length=10)),
            ('port', self.gf('django.db.models.fields.IntegerField')(default=0)),
            ('description', self.gf('django.db.models.fields.TextField')(blank=True)),
        ))
        db.send_create_signal('conntrackt', ['Communication'])


    def backwards(self, orm):
        # Deleting model 'Project'
        db.delete_table('conntrackt_project')

        # Deleting model 'Location'
        db.delete_table('conntrackt_location')

        # Deleting model 'Entity'
        db.delete_table('conntrackt_entity')

        # Deleting model 'Interface'
        db.delete_table('conntrackt_interface')

        # Deleting model 'Communication'
        db.delete_table('conntrackt_communication')


    models = {
        'conntrackt.communication': {
            'Meta': {'object_name': 'Communication'},
            'description': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'destination': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'destination_set'", 'to': "orm['conntrackt.Interface']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'port': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'protocol': ('django.db.models.fields.CharField', [], {'max_length': '10'}),
            'source': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'source_set'", 'to': "orm['conntrackt.Interface']"})
        },
        'conntrackt.entity': {
            'Meta': {'object_name': 'Entity'},
            'description': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'location': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['conntrackt.Location']"}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'project': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['conntrackt.Project']"})
        },
        'conntrackt.interface': {
            'Meta': {'object_name': 'Interface'},
            'address': ('django.db.models.fields.IPAddressField', [], {'max_length': '15'}),
            'description': ('django.db.models.fields.TextField', [], {'default': "'Main network interface.'", 'blank': 'True'}),
            'entity': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['conntrackt.Entity']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'default': "'eth0'", 'max_length': '100'}),
            'netmask': ('django.db.models.fields.IPAddressField', [], {'default': "'255.255.255.255'", 'max_length': '15'})
        },
        'conntrackt.location': {
            'Meta': {'object_name': 'Location'},
            'description': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'conntrackt.project': {
            'Meta': {'object_name': 'Project'},
            'description': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        }
    }

    complete_apps = ['conntrackt']


# -*- coding: utf-8 -*-
#
# Copyright (C) 2013 Branko Majic
#
# This file is part of Django Conntrackt.
#
# Django Conntrackt is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# Django Conntrackt is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# Django Conntrackt.  If not, see <http://www.gnu.org/licenses/>.
#


# Django imports.
from django.forms import ModelForm

# Application imports.
from .models import Project, Location, Entity, Interface, Communication


class WidgetCSSClassFormMixin(object):
    """
    Helper form mixin that can be used for applying additional custom CSS
    classes to form field widgets.

    The mixin accepts the following class options:

        widget_css_classes - Dictionary describing which additional CSS classes
        should be applied to which form widget. Dictinoary keys should be equal
        to form widget names, while the value should be a string containing the
        extra CSS classes that should be applied to it. In order to apply the
        CSS classes to every widget of the form, use the key "ALL"
    """

    def __init__(self, *args, **kwargs):
        """
        Assigns the custom CSS classes form widgets, as configured by the
        widget_css_classes property.
        """

        super(WidgetCSSClassFormMixin, self).__init__(*args, **kwargs)

        for field_name, css_class in self.widget_css_classes.iteritems():
            if field_name == "ALL":
                for field in self.fields.values():
                    if "class" in field.widget.attrs:
                        field.widget.attrs["class"] += " " + css_class
                    else:
                        field.widget.attrs["class"] = css_class
            else:
                field = self.fields[field_name]
                if "class" in field.widget.attrs:
                    field.widget.attrs["class"] += " " + css_class
                else:
                    field.widget.attrs["class"] = css_class


class PlaceholderFormMixin(object):
    """
    Helper form mixin that can be used to set-up placeholders for text widgets.

    The mixin accepts the following class options:

        widget_placeholders - Dictionary describing which placeholders should be
        applied to which widgets. Keys should be equal to form widget names,
        while the values should be the strings that should be set as
        placeholders.
    """

    def __init__(self, *args, **kwargs):
        """
        Assigns the placeholders to text form widgets, as configured by the
        widget_placeholders property.
        """

        super(PlaceholderFormMixin, self).__init__(*args, **kwargs)

        for field_name, placeholder in self.widget_placeholders.iteritems():
            self.fields[field_name].widget.attrs["placeholder"] = placeholder


class EntityForm(WidgetCSSClassFormMixin, PlaceholderFormMixin, ModelForm):
    """
    Implements a custom model form for entities with some styling changes.
    """

    class Meta:
        model = Entity

    widget_placeholders = {"name": "Entity name",
                           "description": "Entity description"}
    widget_css_classes = {"ALL": "span6"}


class InterfaceForm(WidgetCSSClassFormMixin, PlaceholderFormMixin, ModelForm):
    """
    Implements a custom model form for interfaces with some styling changes.
    """

    class Meta:
        model = Interface

    widget_placeholders = {"name": "Interface name",
                           "description": "Interface description",
                           "address": "IP address of interface",
                           "netmask": "IP address netmask"}

    widget_css_classes = {"ALL": "span6"}


class CommunicationForm(WidgetCSSClassFormMixin, PlaceholderFormMixin, ModelForm):
    """
    Implements a custom model form for communications with some styling changes.
    """

    class Meta:
        model = Communication

    widget_placeholders = {"port": "Port used for communication",
                           "description": "Communication description"}

    widget_css_classes = {"ALL": "span6"}


class ProjectForm(WidgetCSSClassFormMixin, PlaceholderFormMixin, ModelForm):
    """
    Implements a custom model form for projects with some styling changes.
    """

    class Meta:
        model = Project

    widget_placeholders = {"name": "Project name",
                           "description": "Project description"}

    widget_css_classes = {"ALL": "span6"}


class LocationForm(WidgetCSSClassFormMixin, PlaceholderFormMixin, ModelForm):
    """
    Implements a custom model form for projects with some styling changes.
    """

    class Meta:
        model = Location

    widget_placeholders = {"name": "Location name",
                           "description": "Location description"}

    widget_css_classes = {"ALL": "span6"}



# -*- coding: utf-8 -*-
#
# Copyright (C) 2013 Branko Majic
#
# This file is part of Django Conntrackt.
#
# Django Conntrackt is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# Django Conntrackt is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# Django Conntrackt.  If not, see <http://www.gnu.org/licenses/>.
#


# Python imports.
from operator import attrgetter


class Rule(object):
    """
    Class representing a single iptables rule. The representation does not
    include the chain to which it is applied, or how it is applied (append vs
    insert).

    The instances of this class can be used for easily outputting a single rule.
    """
    def __init__(self, source, destination, protocol, port, description=""):
        """
        Initialises a rule instance. Sets-up the necessary information passed
        with arguments.

        Arguments:

        source - Source of communication, in format IP/NETMASK.

        destination - Destination of communication, in format IP/NETMASK.

        protocol - Protocol used by the rule. Currently supported values are
        UDP, TCP, and ICMP.

        port - Destination port in a rule if protocol specified is UDP or
        TCP. In case of ICMP this should be the ICMP packet type.

        description - Rule description. This can be used by other classes to
        produce comments in the output.
        """

        if protocol.upper() not in ("TCP", "UDP", "ICMP"):
            raise ValueError("Unsupported protocol specified: %s" % protocol)

        self.source = source
        self.destination = destination
        self.protocol = protocol.lower()
        self.port = port
        self.description = description

    def __unicode__(self):
        """
        Creates string representation of the rule. The format is:

        -s SOURCE -d DESTINATION -p PROTOCOL -m PROTOCOL (--dport|--icmp-type) PORT -j ACCEPT

        Returns:

        String representation of the rule.
        """
        if self.protocol.upper() in ("TCP", "UDP"):
            return "-s %s -d %s -p %s -m %s --dport %s -j ACCEPT" % (self.source, self.destination, self.protocol, self.protocol, self.port)
        elif self.protocol.upper() in ("ICMP"):
            return "-s %s -d %s -p %s -m %s --icmp-type %s -j ACCEPT" % (self.source, self.destination, self.protocol, self.protocol, self.port)

    def __str__(self):
        """
        Creates string representation of the rule. Calls the __unicode__
        function.

        Returns:

        String representation of the rule.
        """

        return self.__unicode__()


class LoopbackRule(object):
    """
    Static iptables rule that accepts all traffic on loopback interface.
    """

    def __init__(self):
        """
        Initialises the rule properties. Sets a static description.
        """

        self.description = "Accept all incoming traffic on loopback interface."

    def __unicode__(self):
        """
        Creates string representation of the rule. The format is:

        -i lo -j ACCEPT

        Returns:

        String representation of the rule.
        """

        return "-i lo -j ACCEPT"

    def __str__(self):
        """
        Creates string representation of the rule. Calls the __unicode__
        function.

        Returns:

        String representation of the rule.
        """

        return self.__unicode__()


class RelatedRule(object):
    """
    Static iptables rule that accepts all related traffic.
    """

    def __init__(self):
        """
        Initialises the rule properties. Sets a static description.
        """

        self.description = "Accept all incoming related traffic."

    def __unicode__(self):
        """
        Creates string representation of the rule. The format is:

        -m state --state RELATED,ESTABLISHED -j ACCEPT

        Returns:

        String representation of the rule.
        """

        return "-m state --state RELATED,ESTABLISHED -j ACCEPT"

    def __str__(self):
        """
        Creates string representation of the rule. Calls the __unicode__
        function.

        Returns:

        String representation of the rule.
        """

        return self.__unicode__()


class Chain(object):
    """
    Class representing a full iptables chain. Every chain has a name, default
    target, and contains a number of rules.

    The instances of this class can be used for easily outputting iptables rules
    for a single chain.
    """

    def __init__(self, name, default):
        """
        Initialises a chain instance. Sets-up the necessary information passed
        through the arguments.

        Arguments:

        name - The chain name (for example, INPUT, OUTPUT, FORWARD).

        default - Default target. Currently supported values are ACCEPT, DROP,
        and REJECT.
        """

        if default not in ("ACCEPT", "DROP", "REJECT"):
            raise ValueError("Unsupported default target specified: %s" % default)

        self.name = name
        self.default = default
        self.rules = []

    def add_rule(self, rule):
        """
        Adds a new rule to the chain.

        Arguments:

        rule - Instance of Rule that should be added to the chain.
        """

        self.rules.append(rule)

    def __unicode__(self):
        """
        Creates string representing of the chain. The format will be:

        :NAME DEFAULT [0:0]
        # DESC_RULE_1
        -A NAME RULE_1
        # DESC_RULE_2
        -A NAME RULE_2
        ...
        # DESC_RULE_N
        -A NAME RULE_N

        It should be noted that the rules will be grouped by their
        description. Description line is not output if description is empty.
        """

        # Set-up the "header".
        rendering = ":%s %s [0:0]" % (self.name, self.default)

        # Group the rules by description.
        rules = list(self.rules)
        rules.sort(key=attrgetter("description"))

        # Use this property to figure out if we need new line separator.
        previous_description = None

        # Process each rule.
        for rule in rules:
            if rule.description != previous_description:
                rendering += "\n"
                if rule.description:
                    rendering += "# %s\n" % rule.description
                previous_description = rule.description
            rendering += "-A %s %s\n" % (self.name, rule)
        rendering += "\n"
        return rendering

    def __str__(self):
        """
        Creates string representation of the chain. Calls the __unicode__
        function.

        Returns:

        String representation of the chain.
        """

        return self.__unicode__()


class Table(object):
    """
    Class representing a single iptables table (i.e. nat, or filter). Each table
    has a name, and a number of associated chains.

    The instances of this class can be used for easily outputting iptables rules
    for a single table.
    """

    def __init__(self, name):
        """
        Initialises a table instance. Sets-up the necessary information passed
        through the arguments.

        Arguments:

        name - Table name (for example, nat or filter).
        """
        self.name = name
        self.chains = []

    def add_chain(self, chain):
        """
        Adds a new chain to the table.

        Arguments:

        chain - Instance of Chain that should be added to the chain.
        """

        self.chains.append(chain)

    def __unicode__(self):
        """
        Creates string representing of the table. The format will be:

        *NAME
        CHAIN_1
        CHAIN_2
        COMMIT
        """

        rendering = "*%s\n" % self.name

        for chain in self.chains:
            rendering += "%s" % chain

        rendering += "COMMIT\n"

        return rendering

    def __str__(self):
        """
        Creates string representation of the table. Calls the __unicode__
        function.

        Returns:

        String representation of the table.
        """

        return self.__unicode__()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2013 Branko Majic
#
# This file is part of Django Conntrackt.
#
# Django Conntrackt is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# Django Conntrackt is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# Django Conntrackt.  If not, see <http://www.gnu.org/licenses/>.
#


# Django imports.
from django.contrib.admin.util import NestedObjects
from django.core.exceptions import ValidationError
from django.core.urlresolvers import reverse
from django.db import models
from django.db.models.query_utils import Q

# Application imports.
from .utils import list_formatter_callback, get_distinct_colors


class SearchManager(models.Manager):
    """
    Custom model manager that implements search for model instances that contain
    a specific string (search term) in fields "name" or "description".
    """

    def search(self, search_term):
        """
        Performs a search for model instances that contain the provided search
        term in fields "name" or "description". The search is case-insensitive.

        Arguments:
          search_term - String to search the name and description for.

        Returns:
          Query set with model instances that matched the search.
        """

        return self.filter(Q(name__icontains=search_term) | Q(description__icontains=search_term))


class RelatedCollectorMixin(object):
    """
    Implements model mixin for easily obtainning related items of a model
    instance.

    The mixin can be used for obtaining all model objects that are directly or
    indirectly linked to the calling model object (through foreign key
    relationships).

    This mixin is very useful in delete views for warning the user about all of
    the related items that will be deleted if the calling item is deleted as
    well.
    """

    def get_dependant_objects(self):
        """
        Creates a list of model objects that depend (reference), both directly
        and indirectly, on calling model object. The calling model object is
        included as the first element of the list. This method call can be used
        in order to obtain a list of model objects that would get deleted in
        case the calling model object gets deleted.

        Returns:
          Nested list of model objects that depend (reference) calling model
          object.
        """

        collector = NestedObjects(using='default')

        collector.collect([self])

        return collector.nested()

    def get_dependant_objects_representation(self):
        """
        Creates a nested list of object representations that depend (reference),
        both directly and indirectly, calling model object. This method call can
        be used in order to obtain a list of string representations of model
        objects that would get deleted in case the calling model object gets
        deleted.

        The resulting nested list can be shown to the user for
        warning/notification purposes using the unordered_list template tag.

        Each non-list element will be a string generated using the
        conntrackt.utils.list_formatter_callback function.

        Returns:
          Nested list of representations of model objects that depend
          (reference) calling model object.
        """

        collector = NestedObjects(using='default')

        collector.collect([self])

        return collector.nested(list_formatter_callback)


class Project(RelatedCollectorMixin, models.Model):
    """
    Implements a model with information about a project. A project has some
    basic settings, and mainly serves the purpose of grouping entities for
    easier handling and administration.

    Fields:

      name - String denoting the project name.
      description - Free-form description of the project.
    """

    name = models.CharField(max_length=100, unique=True)
    description = models.TextField(blank=True)
    objects = SearchManager()
    deletion_collect_models = ["Entity", "Interface"]

    class Meta:
        permissions = (("view", "Can view information"),)

    def __unicode__(self):
        """
        Returns:
          String representation of a project.
        """

        return self.name

    def get_absolute_url(self):
        """
        Return absolute URL for viewing a single project.
        """

        return reverse("project", kwargs={'pk': self.pk})

    def get_project_communications_summary(self):
        """
        Creates a list of dictionaries where each dictionary provides summary of
        a communication. Each dictionary will contain the following keys:

        - source - Source of the communication.
        - source_color - Color that is associated with the source of
          communication. The color will match with color used in the project
          communications diagram.
        - destination - Destination of the communication.
        - destination_color - Color that is associated with the destination of
          communication. The color will match with color used in the project
          communications diagram.
        - protocol - Protocol used for communication.
        - port - Port used for communication.

        Returns:
          List of dictionaries, where each dictionary describes a single communication.
        """

        # Obtain the list of colors.
        colors = get_distinct_colors(self.entity_set.count())

        # Fetch ID's of each end entity.
        entity_ids = self.entity_set.values_list("pk", flat=True).order_by("pk")

        # Map the entity ID's to generated colors.
        entity_colors = dict(zip(entity_ids, colors))

        # Set-up an empty list where the resulting data will be stored.
        communications = []

        # Process each communication, and add the information to result.
        for communication in Communication.objects.filter(source__entity__project=self).select_related():
            communications.append({"source": communication.source.entity.name,
                                   "source_color": entity_colors[communication.source.entity.pk],
                                   "destination": communication.destination.entity.name,
                                   "destination_color": entity_colors[communication.destination.entity.pk],
                                   "protocol": communication.protocol,
                                   "port": communication.port})

        # Finally return the result.
        return communications


class Location(RelatedCollectorMixin, models.Model):
    """
    Implements a model with information about location. Locations can further be
    assigned to entities, letting the user group different servers and equipment
    based on location.

    Locations are not tied to specific project, and they do not have to be
    actual physical locations. Such generic locations are therefore reusable
    accross multiple projects.

    For example, locations can be:

      - Main site
      - Backup site
      - Disaster recovery site
      - Belgrade
      - Stockholm

    Fields:

      name - String denoting the location name.
      description - Free-form description of a location.
    """

    name = models.CharField(max_length=100, unique=True)
    description = models.TextField(blank=True)

    def __unicode__(self):
        """
        Returns:
          String representation of a location.
        """

        return self.name


class Entity(RelatedCollectorMixin, models.Model):
    """
    Models an entity in a project. An entity can be a server, router, or any
    other piece of networking equipment that has its own IP address.

    Entities can also be used for representing subnets etc. This is useful when
    the communication restrictions need to be applied across a subnet.

    Entities are tied to specific projects and locations.

    Fields:

      name - String denoting the entity name.
      description - Free-form description of an entity.
      project - Foreign key pointing to the project to which the entity
      belongs.
      location - Foreign key pointing to the location at which the entity is
      located.
    """

    name = models.CharField(max_length=100)
    description = models.TextField(blank=True)
    project = models.ForeignKey(Project)
    location = models.ForeignKey(Location)
    objects = SearchManager()

    class Meta:
        # Fix the plural form used by Django.
        verbose_name_plural = 'entities'
        # Enforce uniqueness of entity name in a project.
        unique_together = ("name", "project")

    def __unicode__(self):
        """
        Returns:
          String representation of an entity. This identifier contains name of
          entity, its project name, and location name.
        """

        return "%s (%s - %s)" % (self.name, self.project, self.location)

    def incoming_communications(self):
        """
        Returns:
          List of incoming communications for an entity.
        """

        communications = []

        for interface in self.interface_set.all():
            for communication in interface.destination_set.all():
                communications.append(communication)

        return communications

    def outgoing_communications(self):
        """
        Returns:
          List of outgoing communications for an entity.
        """

        communications = []

        for interface in self.interface_set.all():
            for communication in interface.source_set.all():
                communications.append(communication)

        return communications

    def get_absolute_url(self):
        """
        Return absolute URL for viewing a single entity.
        """

        return reverse("entity", kwargs={'pk': self.pk})

    def clean(self):
        """
        Performs additional validation checks on the submitted data. It will
        verify the following:

          - That entity is not linked to any other entity in case of project
            change.
        """

        # Perform the check if entity is being updated.
        if self.pk:
            # Fetch the old data from database.
            # @TODO: Is it better to do copying during __init__ instead?
            old_object = Entity.objects.get(pk=1)

            # Make sure that entity has no communications in current project if
            # moving it around.
            if self.project != old_object.project and (self.incoming_communications() or self.outgoing_communications()):
                raise ValidationError("The entity cannot be moved to different project as long as it has valid communications with entities in current project.")


class Interface(RelatedCollectorMixin, models.Model):
    """
    Models a representation of an interface on an entity. It can be used for
    representing the subnets as well.

    Each interface is coupled with a specific Entity.

    Fields:
      name - String denoting the interface name. For example 'eth0', 'eth1'
      etc.
      description - Free-form description of an interface.
      entity - Foreign key pointing to the entity to which the interface
      belongs.
      address - IP address of an interface. It's possible to store network
      address in it as well.
      netmask - Netmask of the interface. By default this is /32
      (255.255.255.255), but in case of subnet entities this can be used for
      denoting the network netmask.
    """

    name = models.CharField(max_length=100, default='eth0')
    description = models.TextField(blank=True, default='Main network interface.')
    entity = models.ForeignKey(Entity)
    address = models.IPAddressField()
    netmask = models.IPAddressField(default='255.255.255.255')

    class Meta:
        # Enforce uniqueness of interface name in an entity. Enforce uniqueness
        # of IP address in a subnet for an entity.
        unique_together = (("name", "entity"),
                           ("entity", "address", "netmask"),)

    def __unicode__(self):
        """
        Returns:
          String representation of an interface. In case of single IP this will
          simply be the interface name and IP address. In case of subnet it will
          include the netmask as well.
        """

        if self.netmask == '255.255.255.255':
            return '%s (%s)' % (self.entity.name, self.address)
        else:
            return '%s (%s/%s)' % (self.entity.name, self.address, self.netmask)


class Communication(RelatedCollectorMixin, models.Model):
    """
    Models a representation of allowed network communication. This lets the user
    display the possible network connections that should be allowed. Information
    from the communication instances is also used for generating the iptables
    rules for the entities.

    Communication instances allow the user to specify one of the three possible
    protocols and related information:

      - TCP, along with the TCP port.
      - UDP, along with the UDP port.
      - ICMP, along with the ICMP type.

    Allowed communication is always represented as combination of source
    interface, destination interface, protocol, and port/ICMP type.

    Fields:
      source - Foreign key to the source (originating) interface. The
      communication is expected to come _from_ the source.
      destination - Foreign key to the destination interface. The destination
      interface is expected to be able to accept incoming connections
      (i.e. entity's servers are listening on those).
      protocol - Textual field denoting the protocol that is used for
      communication. This can be 'TCP', 'UDP', or 'ICMP'.
      port - Port number used by the protocol. In case of ICMP, this is an ICMP
      type (in numeric form).
      description - Free-form text that can be used to describe the
      communication. This is also used when generating the iptables rules for
      documenting the rules.
    """

    PROTOCOL_CHOICES = (
        ('TCP', 'TCP'),
        ('UDP', 'UDP'),
        ('ICMP', 'ICMP'),
        )

    source = models.ForeignKey(Interface, related_name='source_set')
    destination = models.ForeignKey(Interface, related_name='destination_set')
    protocol = models.CharField(max_length=10, choices=PROTOCOL_CHOICES)
    port = models.IntegerField(default=0)
    description = models.TextField(blank=True)

    class Meta:
        # Enforce uniqueness of communication.
        unique_together = ("source", "destination", "protocol", "port")

    def __unicode__(self):
        """
        Returns:
          String representation of an interface. This involves showing the
          source and destination _entity_ name, protocol, and port.
        """

        return "%s -> %s (%s:%s)" % (self.source.entity.name, self.destination.entity.name, self.protocol, self.port)

    def clean(self):
        """
        Performs additional validation checks on the submitted data. It will
        verify the following:

          - That source and destination interface belongs to distinct entities.
          - That the specified protocol is supported.
        """

        if self.source.entity == self.destination.entity:
            raise ValidationError('Source and destination entities are identical.')

        if self.source.entity.project != self.destination.entity.project:
            raise ValidationError('Source and destination entities do not belong to the same project')

        if (self.protocol.upper(), self.protocol.upper()) not in self.PROTOCOL_CHOICES:
            raise ValidationError('%s is not a supported protocol.' % self.protocol)

    def edit_link(self):
        """
        This method is used for providing an additional 'Edit' link in the admin
        site for the communication instances (for the display_list).

        This provides ability to let all of the other fields of a communication
        instance to be editable.
        """

        return "Edit"

    def source_representation(self):
        """
        Produces string representation of communication that includes only the
        source interface information.

        The method is useful where the destination context is well known.

        Returns:
            Communication representation that includes only information about
            the source interface.
        """

        return "%s - %s: %d" % (self.source, self.protocol, self.port)

    def destination_representation(self):
        """
        Produces string representation of communication that includes only the
        destination interface information.

        The method is useful where the source context is well known.

        Returns:
            Communication representation that includes only information about
            the destination interface.
        """

        return "%s - %s: %d" % (self.destination, self.protocol, self.port)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2013 Branko Majic
#
# This file is part of Django Conntrackt.
#
# Django Conntrackt is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# Django Conntrackt is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# Django Conntrackt.  If not, see <http://www.gnu.org/licenses/>.
#


# Django imports.
from django.test import TestCase

# Third-party Python library imports.
import palette
import pydot
import mock

# Application imports.
from conntrackt.models import Entity, Project, Communication
from conntrackt import utils
from .factories import setup_test_data


class GenerateEntityIptablesTest(TestCase):

    def setUp(self):
        """
        Set-up some test data.
        """

        setup_test_data()

    def test_generated_iptables(self):
        """
        Tests if the entity's iptables are generated properly or not.
        """

        entity = Entity.objects.get(pk=1)
        generated = utils.generate_entity_iptables(entity)

        expected = """*filter
:INPUT DROP [0:0]
# Accept all incoming related traffic.
-A INPUT -m state --state RELATED,ESTABLISHED -j ACCEPT

# Accept all incoming traffic on loopback interface.
-A INPUT -i lo -j ACCEPT

# Communicate over ICMP:8
-A INPUT -s 192.168.1.2/255.255.255.255 -d 192.168.1.1/255.255.255.255 -p icmp -m icmp --icmp-type 8 -j ACCEPT

# Communicate over TCP:22
-A INPUT -s 192.168.1.2/255.255.255.255 -d 192.168.1.1/255.255.255.255 -p tcp -m tcp --dport 22 -j ACCEPT
-A INPUT -s 10.10.4.0/255.255.255.0 -d 192.168.1.1/255.255.255.255 -p tcp -m tcp --dport 22 -j ACCEPT

# Communicate over TCP:3306
-A INPUT -s 192.168.1.3/255.255.255.255 -d 192.168.1.1/255.255.255.255 -p tcp -m tcp --dport 3306 -j ACCEPT

:OUTPUT ACCEPT [0:0]
:FORWARD DROP [0:0]
COMMIT
*nat
:PREROUTING ACCEPT [0:0]
:INPUT ACCEPT [0:0]
:OUTPUT ACCEPT [0:0]
:POSTROUTING ACCEPT [0:0]
COMMIT
"""
        self.assertEqual(generated, expected)


class GetDistinctColorsTest(TestCase):
    """
    Tests covering the get_distinct_colors function.
    """

    def test_count(self):
        """
        Tests if correct number of distinct colours are returned.
        """

        colors = utils.get_distinct_colors(13)

        self.assertEqual(len(colors), 13)

        colors = utils.get_distinct_colors(123)

        self.assertEqual(len(colors), 123)

    def test_start(self):
        """
        Tests if the passed start colour is returned as part of generated
        colours.
        """

        start = palette.Color("#AA3311")

        colors = utils.get_distinct_colors(10, start)

        self.assertEqual(start.hex, colors[0].hex)

    def test_color_distance(self):
        """
        Tests if the generated colous all have proper distance between
        each-other.
        """

        colors = utils.get_distinct_colors(13)

        # Set allowed margin of difference to 0.1%
        delta = (1 / 13.) * 0.001

        # Calculate diffs between colours.
        diffs = [colors[i + 1].hsl["h"] - colors[i].hsl["h"] for i in range(12)]

        # Take first diff as reference point.
        reference = diffs[0]

        # Create list that contains True/False for diffs depending on whether
        # they're in delta-surrounding of reference point.
        equal = [(abs(diff - reference) < delta) for diff in diffs]

        # There should be 12 True values.
        self.assertEqual(equal.count(True), 12)

        # Check the difference between first and last colour.
        equal = abs(colors[0].hsl["h"] + 1 - colors[12].hsl["h"] - reference) < delta
        self.assertEqual(True, equal)

    def test_zero_count(self):
        """
        Tests if an empty list is returned in case the requested count is 0.
        """

        colors = utils.get_distinct_colors(0)

        self.assertIsInstance(colors, list)
        self.assertEqual(len(colors), 0)

    def test_negative_count(self):
        """
        Tests if proper exception is raised in case a negative count is passed.
        """

        self.assertRaisesRegexp(ValueError, "^Count must be a non-negative integer value\.$", utils.get_distinct_colors, -10)


class GenerateProjectDiagramTest(TestCase):
    """
    Tests the generate_project_diagram function.
    """

    def setUp(self):
        """
        Set-up some test data.
        """

        setup_test_data()

    def test_unique_entity_colors(self):
        """
        Tests if each node/entity in the graph will have a unique colour.
        """

        # Get diagram for project
        project = Project.objects.get(pk=1)
        diagram = utils.generate_project_diagram(project)

        # Extract all nodes
        clusters = diagram.get_subgraphs()
        nodes = []
        for cluster in clusters:
            nodes.extend(cluster.get_nodes())

        # Get the node colours.
        colors = [n.get_color() for n in nodes]

        # Verify they're all unique colours.
        self.assertEqual(len(colors), len(set(colors)))

    def test_edge_colours(self):
        """
        Tests if the edge colours match with source node/entity colour.
        """

        # Get diagram for project
        project = Project.objects.get(pk=1)
        diagram = utils.generate_project_diagram(project)

        # Extract all nodes and edges.
        clusters = diagram.get_subgraphs()
        nodes = {}
        for cluster in clusters:
            for node in cluster.get_nodes():
                nodes[node.get_name()] = node
        edges = diagram.get_edges()

        # Validate that edges have same colour as the source nodes.
        for edge in edges:
            self.assertEqual(nodes[edge.get_source()].get_color(), edge.get_color())

    def test_entities_present(self):
        """
        Tests if all (and only) specific project entities are in the graph.
        """

        # Get diagram for project
        project = Project.objects.get(pk=1)
        diagram = utils.generate_project_diagram(project)

        # Set-up expected node names.
        expected_node_names = [u"Test Entity 1", u"Test Entity 2", u"Test Entity 3", u"Test Subnet 4"]

        # Get all nodes from diagram.
        clusters = diagram.get_subgraphs()
        nodes = []
        for cluster in clusters:
            nodes.extend(cluster.get_nodes())

        # Get the node names, strip the quotes from them.
        node_names = [n.get_name().replace('"', '') for n in nodes]

        # Validate that the two lists contain same elements.
        self.assertEqual(sorted(expected_node_names), sorted(node_names))

    def test_communications_present(self):
        """
        Tests if all (and only) specific project communications are in the
        graph.
        """

        # Get diagram for project
        project = Project.objects.get(pk=1)
        diagram = utils.generate_project_diagram(project)

        # Get all edges from the diagram.
        edges = diagram.get_edges()

        # Create list of edge labels.
        edge_labels = ["%s -> %s (%s)" % (e.get_source().replace('"', ''),
                                           e.get_destination().replace('"', ''),
                                           e.get_label().replace('"', '')) for e in edges]

        # Create list of expected edge labels
        expected_edge_labels = [u'Test Entity 1 -> Test Entity 2 (UDP:123)',
                                u'Test Entity 1 -> Test Entity 3 (UDP:53)',
                                u'Test Entity 2 -> Test Entity 1 (ICMP:8)',
                                u'Test Entity 2 -> Test Entity 1 (TCP:22)',
                                u'Test Entity 3 -> Test Entity 1 (TCP:3306)',
                                u'Test Subnet 4 -> Test Entity 1 (TCP:22)']

        self.assertEqual(sorted(expected_edge_labels), sorted(edge_labels))

    def test_locations_present(self):
        """
        Tests if all (and only) specific project locations are in the graph (as
        clusters).
        """

        # Get diagram for project.
        project = Project.objects.get(pk=1)
        diagram = utils.generate_project_diagram(project)

        # Set-up expected cluster names (based on locations).
        expected_cluster_names = ["cluster_test_location_1", "cluster_test_location_2"]

        # Get cluster names.
        cluster_names = [s.get_name() for s in diagram.get_subgraphs()]

        self.assertEqual(sorted(expected_cluster_names), sorted(cluster_names))

    def test_return_type(self):
        """
        Tests if a correct object type is returned.
        """

        # Get diagram for project.
        project = Project.objects.get(pk=1)
        diagram = utils.generate_project_diagram(project)

        self.assertEqual(type(diagram), pydot.Dot)

    def test_graph_properties(self):
        """
        Tests if graph properties have been set-up properly.
        """

        # Get diagram for project.
        project = Project.objects.get(pk=1)
        diagram = utils.generate_project_diagram(project)

        self.assertEqual("digraph", diagram.get_graph_type())
        self.assertEqual("transparent", diagram.get_bgcolor())
        self.assertEqual("1.5", diagram.get_nodesep())
        self.assertEqual([{"shape": "record"}], diagram.get_node_defaults())


class ListFormatterCallbackTest(TestCase):
    """
    Tests the list_formatter_callback function.
    """

    def test_get_absolute_url(self):
        """
        Test the return result in case the get_absolute_url is available on
        passed object instance.
        """

        # Set-up a minimal object mock.
        obj = mock.Mock(spec=["_meta", "__repr__", "get_absolute_url"])
        obj._meta = mock.Mock()
        obj._meta.verbose_name = "name"
        obj.__repr__ = mock.Mock()
        obj.__repr__.return_value = "representation"
        obj.get_absolute_url.return_value = "url"

        self.assertEqual(utils.list_formatter_callback(obj), '<strong>Name</strong>: <a href="url">representation</a>')

    def test_no_get_absolute_url(self):
        """
        Test the return result in case the get_absolute_url is not available on
        passed object instance.
        """

        # Set-up a minimal object mock.
        obj = mock.Mock(spec=["_meta", "__repr__"])
        obj._meta = mock.Mock()
        obj._meta.verbose_name = "name"
        obj.__repr__ = mock.Mock()
        obj.__repr__.return_value = "representation"

        self.assertEqual(utils.list_formatter_callback(obj), "<strong>Name</strong>: representation")



# -*- coding: utf-8 -*-
#
# Copyright (C) 2013 Branko Majic
#
# This file is part of Django Conntrackt.
#
# Django Conntrackt is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# Django Conntrackt is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# Django Conntrackt.  If not, see <http://www.gnu.org/licenses/>.
#


# Standard library imports.
import re
import itertools

# Third-party Python library imports.
import palette
import pydot

# Django imports.
from django.template import Context, loader
from django.utils.html import format_html
from django.utils.text import capfirst

# Application imports.
import iptables
import models


def generate_entity_iptables(entity):
    """
    Generates full iptables rules for the supplied entity. The generated rules
    can be fed directly to the iptables-restore utility.

    Arguments:

        entity - An Entity instance for which the iptables rules should be
        generated.

    Returns:

        String containing the iptables rules for entity.
    """

    # Fetch list of incoming communications.
    incoming = entity.incoming_communications()

    # Set-up the nat table.
    nat = iptables.Table("nat")
    for chain in ("PREROUTING", "INPUT", "OUTPUT", "POSTROUTING"):
        nat.add_chain(iptables.Chain(chain, "ACCEPT"))

    # Set-up the filter table INPUT chain.
    filter = iptables.Table("filter")
    input = iptables.Chain("INPUT", "DROP")

    input.add_rule(iptables.LoopbackRule())
    input.add_rule(iptables.RelatedRule())

    for communication in incoming:
        source = "%s/%s" % (communication.source.address, communication.source.netmask)
        destination = "%s/%s" % (communication.destination.address, communication.destination.netmask)
        input.add_rule(iptables.Rule(source, destination, communication.protocol, communication.port, communication.description))
    filter.add_chain(input)

    # Set-up empty chains.
    filter.add_chain(iptables.Chain("OUTPUT", "ACCEPT"))
    filter.add_chain(iptables.Chain("FORWARD", "DROP"))

    # Construct the iptables file using the two tables.
    content = "%s%s" % (filter, nat)

    return content


def get_distinct_colors(count, start=palette.Color("#AE1111")):
    """
    Generates a number of distinct colours, and returns them as a list. The
    colours are generated using the HSL (hue, saturation, lightness) model,
    where saturation and lightness is kept the same for all colours, with
    differing hue. The hue difference between each subsequent color in the list
    is kept the same.

    Arguments:

        count - Total number of colours that should be generated.

        start - First colour that should be taken as a start point. All colours
        are generated relative to this colour by increasing the hue. Should be
        an instance of palette.Color class. Defaults to RGB colour "#AE1111".

    Return:

        List of distinct palette.Color instances.
    """

    # If zero is passed as count, return empty list.
    if count == 0:
        return []
    elif count < 0:
        raise ValueError("Count must be a non-negative integer value.")

    # Read the HSL from provided Color.
    hue, sat, lum = start.hsl["h"], start.hsl["s"], start.hsl["l"]

    # Calculate the step increase.
    step = 1 / float(count)

    # Initiate an empty list that will store the generated colours.
    colors = []

    # Generate new colour by increasing the hue as long as we haven't generated
    # the requested number of colours.
    while len(colors) < count:
        colors.append(palette.Color(hsl=(hue, sat, lum)))
        hue += step

    return colors


def generate_project_diagram(project):
    """
    Generates communication diagram for provided project.

    Arguments:

        project - Project for which the diagram should be generated. Instance of
        conntrackt.models.Project class.

    Returns:

        Dot diagram (digraph) representing all of the communications in a
        project.
    """

    # Set-up the graph object.
    graph = pydot.Dot(graph_name=project.name, graph_type="digraph", bgcolor="transparent", nodesep="1.5")
    # Set-up defaults for the graph nodes.
    graph.set_node_defaults(shape="record")

    # Obtain list of all entities in a project.
    entities = project.entity_set.all()

    # Set-up dictinary that will contains clusters of entities belonging to same
    # location.
    clusters = {}

    # Dictinoary for storing mapping between nodes and colours.
    node_colors = {}

    # Get distinct colours, one for each node/entity.
    colors = get_distinct_colors(entities.count())

    # Create nodes based on entities, and put them into correct cluster.
    for entity in entities.order_by("pk"):

        # Try to get the existing cluster based on location name.
        location = entity.location
        cluster_name = location.name.replace(" ", "_").lower()
        cluster = clusters.get(cluster_name, None)

        # Set-up a new cluster for location encountered for the first time.
        if cluster is None:
            cluster = pydot.Cluster(graph_name=cluster_name, label=location.name)
            clusters[cluster_name] = cluster

        # Fetch a colour that will be associated with the node/entity.
        node_color = colors.pop(0)
        node_colors[entity.id] = node_color.hex

        # Determine whether the node label should be black or white based on brightness of node colour.
        node_color_brightness = 1 - (node_color.rgb["r"] * 0.299 + node_color.rgb["g"] * 0.587 + node_color.rgb["b"] * 0.114)

        if node_color_brightness < 0.5:
            font_color = "black"
        else:
            font_color = "white"

        # Finally create the node, and add it to location cluster.
        node = pydot.Node(entity.name, style="filled", color=node_color.hex, fontcolor=font_color)
        cluster.add_node(node)

    # Add clusters to the graph.
    for cluster in clusters.values():
        graph.add_subgraph(cluster)

    # Get all project communications.
    communications = models.Communication.objects.filter(source__entity__project=project)

    # Add the edges (lines) representing communications, drawing them with same
    # colour as the source node/entity.
    for comm in communications:
        edge_color = node_colors[comm.source.entity.id]

        label = '"%s:%s"' % (comm.protocol, str(comm.port))

        edge = pydot.Edge(comm.source.entity.name, comm.destination.entity.name, label=label, color=edge_color)

        graph.add_edge(edge)

    return graph


def list_formatter_callback(obj):
    """
    Creates model object representation in format:

    MODEL_NAME: OBJECT_REPRESENTATION

    If passed object has a callable get_absolute_url method, the
    instance representation will be surrouned by an HTML anchor
    (<a></a>) where target is set to value of the get_absolute_url()
    method call.

    Arguments:
      obj - Model object whose representation should be returned.

    Returns:
      String represenation of passed model object.
    """

    try:
        return format_html('<strong>{0}</strong>: <a href="{1}">{2}</a>', capfirst(obj._meta.verbose_name), obj.get_absolute_url(), str(obj))
    except AttributeError:
        return format_html('<strong>{0}</strong>: {1}', capfirst(obj._meta.verbose_name), str(obj))



# -*- coding: utf-8 -*-
#
# Copyright (C) 2013 Branko Majic
#
# This file is part of Django Conntrackt.
#
# Django Conntrackt is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# Django Conntrackt is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# Django Conntrackt.  If not, see <http://www.gnu.org/licenses/>.
#


# Application imports.
from conntrackt.views import RedirectToNextMixin


class StaticSuccessUrlFakeMixin(object):
    """
    Helper view for testing the RedirectToNextMixinView mixin.
    """

    def get_success_url(self):
        return self.success_url


class RedirectToNextMixinView(RedirectToNextMixin, StaticSuccessUrlFakeMixin):
    """
    Helper view for testing the RedirectToNextMixinView mixin. StaticSuccessUrl
    is there just to provide default for get_success_url().
    """

    success_url = "/STATIC"

    def __init__(self, request):
        """
        Initialise the request to provided value.
        """

        self.request = request



# -*- coding: utf-8 -*-
#
# Copyright (C) 2013 Branko Majic
#
# This file is part of Django Conntrackt.
#
# Django Conntrackt is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# Django Conntrackt is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# Django Conntrackt.  If not, see <http://www.gnu.org/licenses/>.
#


# Standard library imports.
from StringIO import StringIO
from zipfile import ZipFile, ZIP_DEFLATED
import json

# Django imports.
from django.contrib.auth.decorators import permission_required
from django.contrib import messages
from django.core.exceptions import ValidationError
from django.core.urlresolvers import reverse, reverse_lazy
from django.db.models import Q
from django.db.models.deletion import Collector
from django.http import HttpResponse
from django.shortcuts import render_to_response, get_object_or_404
from django.views.generic import TemplateView, DetailView, CreateView, UpdateView, DeleteView, View

# Third-party application imports.
from braces.views import MultiplePermissionsRequiredMixin, SetHeadlineMixin

# Application imports.
from .forms import ProjectForm, LocationForm, EntityForm, InterfaceForm, CommunicationForm
from .models import Project, Entity, Location, Interface, Communication
from .utils import generate_entity_iptables, generate_project_diagram


class RedirectToNextMixin(object):
    """
    View mixin that can be used for redirecting the user to URL defined through
    a GET parameter. The mixin is usable with Create/Update/Delete views that
    utilise the get_success_url() call.

    The mixin accepts the following class options:

        next_parameter - Name of the GET parameter that contains the redirect
        URL. Defaults to "next".
    """

    next_parameter = "next"

    def get_success_url(self):
        """
        Returns the success URL to which the user will be redirected.
        """

        return self.request.GET.get(self.next_parameter, super(RedirectToNextMixin, self).get_success_url())


class RelatedItemsMixin(object):
    """
    View mixin that adds related items of a referenced model object to context
    data in form of a nested list, including the reference model object as the
    first element of a list.

    The context data will be passed using the "related_items" key.

    This data can be used in the template by passing it through the
    unordered_list template tag.

    The reference object is accessed via "object" property of calling view
    (i.e. self.object).

    For more details on implementation, see:

    RelatedCollectorMixin.get_dependant_objects_representation method
    """

    def get_context_data(self, **kwargs):
        """
        Adds the related items of a reference model object to context data.

        Returns:
          Context data.
        """

        # Set the context using the parent class.
        context = super(RelatedItemsMixin, self).get_context_data(**kwargs)

        # Add to context the nested list of string representations of related
        # items.
        context["related_items"] = self.object.get_dependant_objects_representation()

        return context


class IndexView(MultiplePermissionsRequiredMixin, TemplateView):
    """
    Custom view used for rendering the index page.
    """

    template_name = 'conntrackt/index.html'

    # Required permissions.
    permissions = {
        "all": ("conntrackt.view",),
        }
    # Raise authorisation denied exception for unmet permissions.
    raise_exception = True

    def get_context_data(self, **kwargs):
        """
        Returns the context data that should be used for rendering of the
        template.

        Adds the 'projects' context object containing all of the projects
        available sorted aplhabetically by name.
        """

        # Set the context using the parent class.
        context = super(IndexView, self).get_context_data(**kwargs)

        # Store information about all projcts in context.
        context['projects'] = Project.objects.all().order_by('name')

        # Store information about all locations in context.
        context['locations'] = Location.objects.all().order_by('name')

        return context


class ProjectView(MultiplePermissionsRequiredMixin, DetailView):
    """
    Custom view for presenting the project information.
    """

    model = Project

    permissions = {
        "all": ("conntrackt.view",),
        }
    # Raise authorisation denied exception for unmet permissions.
    raise_exception = True

    def get_context_data(self, **kwargs):
        """
        Returns the context data that should be used for rendering of the
        template.

        Adds the 'location_entities' context object that contains tuples of the
        form '(location, entities)', where location is an instance of a
        Location, and entities is a query set of entities that belong to that
        particular location, and to related project.
        """

        # Set the context using the parent class.
        context = super(ProjectView, self).get_context_data(**kwargs)

        # Set-up an array that will contaion (location, entities) tuples.
        location_entities = []

        # Add the (location, entities) tuple for each location that has entities
        # belonging to the related project.
        for location in Location.objects.filter(entity__project=self.object).distinct().order_by("name"):
            entities = Entity.objects.filter(project=self.object, location=location)
            location_entities.append((location, entities))

        # Add the (location, entities) tuples to context.
        context['location_entities'] = location_entities

        # Add all project communications to context.
        context['communications'] = self.object.get_project_communications_summary()

        # Finally return the context.
        return context


class EntityView(MultiplePermissionsRequiredMixin, DetailView):
    """
    Custom view for presenting entity information.
    """

    # Optimise the query to fetch the related data from reverse relationships.
    queryset = Entity.objects.all()
    queryset = queryset.prefetch_related('interface_set__destination_set__source__entity')
    queryset = queryset.prefetch_related('interface_set__destination_set__destination__entity')
    queryset = queryset.prefetch_related('interface_set__source_set__source__entity')
    queryset = queryset.prefetch_related('interface_set__source_set__destination__entity')

    # Required permissions.
    permissions = {
        "all": ("conntrackt.view",),
        }
    # Raise authorisation denied exception for unmet permissions.
    raise_exception = True

    def get_context_data(self, **kwargs):
        """
        Returns the context data that should be used for rendering of the
        template.

        Adds the 'entity_iptables' context object that contains full iptables
        rules generated for the entity.
        """

        # Call the parent class method.
        context = super(EntityView, self).get_context_data(**kwargs)

        # Add the rendered iptables rules to the context.
        context['entity_iptables'] = generate_entity_iptables(self.object)

        # Add the incoming and outgoing commmunication to the context.
        context["incoming_communications"] = self.object.incoming_communications()
        context["outgoing_communications"] = self.object.outgoing_communications()

        # Add the interfaces to the context.
        context["interfaces"] = self.object.interface_set.all().order_by("name")

        # Add project/location to the context.
        context["project"] = self.object.project
        context["location"] = self.object.location

        return context


@permission_required("conntrackt.view", raise_exception=True)
def entity_iptables(request, pk):
    """
    Custom view that returns response containing iptables rules generated for an
    entity.

    Makes sure to set the Content-Disposition of a response in order to
    signal the browser it should start download of this view's response
    immediately. Also sets the suggested filename for it.

    Arguments:

        pk - Primary key of the Entity object for which the rules should be
        generated.

    Returns:

        Response object that contains the iptables rules for specified entity.
    """

    # Fetch the entity, and construct the response with iptables rules as
    # content.
    entity = get_object_or_404(Entity, pk=pk)
    content = generate_entity_iptables(entity)
    response = HttpResponse(content, mimetype='text/plain')

    # Add the Content-Disposition information for the browser, telling the
    # browser to download the file with suggested filename.
    response['Content-Disposition'] = "attachment; filename=%s-iptables.conf" % entity.name.lower().replace(" ", "_")

    return response


@permission_required("conntrackt.view", raise_exception=True)
def project_iptables(request, project_id, location_id=None):
    """
    Custom view for obtaining iptables for all entities of a project or project
    location in a single ZIP file.

    Arguments:

        request - Request object.

        project_id - Unique ID of the project for whose entities the iptables
        rules should be generated.

        location_id - Optional unique ID of the project location for whose
        entities the iptables rules should be generated. Default is None, which
        means generate rules for _all_ entities in a project.

    Returns:

        Response object that contains the ZIP file and Content-Disposition
        information.
    """

    # Fetch the project.
    project = get_object_or_404(Project, pk=project_id)

    # Set-up a string IO object to which we'll write the ZIP file (in-memory).
    buff = StringIO()

    # Create a new ZIP file in-memory.
    zipped_iptables = ZipFile(buff, "w", ZIP_DEFLATED)

    # Create the response object, setting the mime type so browser could offer
    # to open the file with program as well.
    response = HttpResponse(mimetype='application/zip')

    # If specific location was specified, get the entities that are part of that
    # project location only, otherwise fetch all of the project's entities. Also
    # set-up the filename that will be suggested to the browser.
    if location_id:
        location = get_object_or_404(Location, pk=location_id)
        entities = project.entity_set.filter(location=location)
        filename = '%s-%s-iptables.zip' % (project.name.lower().replace(" ", "_"), location.name.lower().replace(" ", "_"))
    else:
        entities = project.entity_set.all()
        filename = '%s-iptables.zip' % (project.name.lower().replace(" ", "_"))

    # Render iptables rules for each entity, placing them in the ZIP archive.
    for entity in entities:
        entity_iptables = generate_entity_iptables(entity)
        zipped_iptables.writestr("%s-iptables.conf" % entity.name.lower().replace(" ", "_"), entity_iptables)

    # Close the archive, and flush the buffer.
    zipped_iptables.close()
    buff.flush()

    # Write the contents of our buffer (ZIP archive) to response content, and
    # close the IO string.
    response.write(buff.getvalue())
    buff.close()

    # Set the Content-Disposition so the browser would know it should download
    # the archive, and suggest the filename.
    response['Content-Disposition'] = 'attachment; filename="%s"' % filename

    # Finally return the response object.
    return response


class ProjectCreateView(RedirectToNextMixin, SetHeadlineMixin, MultiplePermissionsRequiredMixin, CreateView):
    """
    View for creating a new project.
    """

    model = Project
    form_class = ProjectForm
    headline = "Add new project"
    template_name = "conntrackt/create_form.html"

    # Required permissions.
    permissions = {
        "all": ("conntrackt.add_project",),
        }

    # Raise authorisation denied exception for unmet permissions.
    raise_exception = True


class ProjectUpdateView(RedirectToNextMixin, SetHeadlineMixin, MultiplePermissionsRequiredMixin, UpdateView):
    """
    View for modifying an existing project.
    """

    model = Project
    form_class = ProjectForm
    template_name = "conntrackt/update_form.html"

    # Required permissions.
    permissions = {
        "all": ("conntrackt.change_project",),
        }

    # Raise authorisation denied exception for unmet permissions.
    raise_exception = True

    def get_headline(self):
        """
        Set headline based on project name.
        """

        return "Update project %s" % self.object.name


class ProjectDeleteView(RelatedItemsMixin, RedirectToNextMixin, SetHeadlineMixin, MultiplePermissionsRequiredMixin, DeleteView):
    """
    View for deleting a project.
    """

    model = Project
    template_name = "conntrackt/delete_form.html"

    # Required permissions.
    permissions = {
        "all": ("conntrackt.delete_project",),
        }

    # Raise authorisation denied exception for unmet permissions.
    raise_exception = True

    success_url = reverse_lazy("index")

    def post(self, *args, **kwargs):
        """
        Add a success message that will be displayed to the user to confirm the
        project deletion.
        """

        messages.success(self.request, "Project %s has been removed." % self.get_object().name, extra_tags="alert alert-success")

        return super(ProjectDeleteView, self).post(*args, **kwargs)

    def get_headline(self):
        """
        Set headline based on project name.
        """

        return "Delete project %s" % self.object.name


class LocationCreateView(RedirectToNextMixin, SetHeadlineMixin, MultiplePermissionsRequiredMixin, CreateView):
    """
    View for creating a new location.
    """

    model = Location
    form_class = LocationForm
    headline = "Add new location"
    template_name = "conntrackt/create_form.html"

    # Required permissions.
    permissions = {
        "all": ("conntrackt.add_location",),
        }

    # Raise authorisation denied exception for unmet permissions.
    raise_exception = True

    success_url = reverse_lazy("index")


class LocationUpdateView(RedirectToNextMixin, SetHeadlineMixin, MultiplePermissionsRequiredMixin, UpdateView):
    """
    View for modifying an existing location.
    """

    model = Location
    form_class = LocationForm
    template_name = "conntrackt/update_form.html"

    # Required permissions.
    permissions = {
        "all": ("conntrackt.change_location",),
        }

    # Raise authorisation denied exception for unmet permissions.
    raise_exception = True

    success_url = reverse_lazy("index")

    def get_headline(self):
        """
        Set headline based on location name.
        """

        return "Update location %s" % self.object.name


class LocationDeleteView(RelatedItemsMixin, RedirectToNextMixin, SetHeadlineMixin, MultiplePermissionsRequiredMixin, DeleteView):
    """
    View for deleting a location.
    """

    model = Location
    template_name = "conntrackt/delete_form.html"

    # Required permissions.
    permissions = {
        "all": ("conntrackt.delete_location",),
        }

    # Raise authorisation denied exception for unmet permissions.
    raise_exception = True

    success_url = reverse_lazy("index")

    def post(self, *args, **kwargs):
        """
        Add a success message that will be displayed to the user to confirm the
        location deletion.
        """

        messages.success(self.request, "Location %s has been removed." % self.get_object().name, extra_tags="alert alert-success")

        return super(LocationDeleteView, self).post(*args, **kwargs)

    def get_headline(self):
        """
        Set headline based on location name.
        """

        return "Delete location %s" % self.object.name


class EntityCreateView(RedirectToNextMixin, SetHeadlineMixin, MultiplePermissionsRequiredMixin, CreateView):
    """
    View for creating a new entity.
    """

    model = Entity
    form_class = EntityForm
    headline = "Add new entity"
    template_name = "conntrackt/create_form.html"

    # Required permissions.
    permissions = {
        "all": ("conntrackt.add_entity",),
        }

    # Raise authorisation denied exception for unmet permissions.
    raise_exception = True

    def get_form(self, form_class):
        """
        Returns an instance of form that can be used by the view.

        The method will limit the project or location select inputs if request
        contained this information.
        """

        form = super(EntityCreateView, self).get_form(form_class)

        # Limit the project selection if required.
        project_id = self.request.GET.get("project", None)
        if project_id:
            form.fields["project"].queryset = Project.objects.filter(pk=project_id)
            form.fields["project"].empty_label = None

        # Limit the location selection if required.
        location_id = self.request.GET.get("location", None)
        if location_id:
            form.fields["location"].queryset = Location.objects.filter(pk=location_id)
            form.fields["location"].empty_label = None

        return form

    def get_initial(self):
        """
        Returns initial values that should be pre-selected (if they were
        specified through a GET parameter).
        """

        initial = super(EntityCreateView, self).get_initial()

        initial["project"] = self.request.GET.get("project", None)
        initial["location"] = self.request.GET.get("location", None)

        return initial


class EntityUpdateView(RedirectToNextMixin, SetHeadlineMixin, MultiplePermissionsRequiredMixin, UpdateView):
    """
    View for updating an existing entity.
    """

    model = Entity
    form_class = EntityForm
    template_name = "conntrackt/update_form.html"

    # Required permissions.
    permissions = {
        "all": ("conntrackt.change_entity",),
        }

    # Raise authorisation denied exception for unmet permissions.
    raise_exception = True

    def get_headline(self):
        """
        Set headline based on entity name.
        """

        return "Update entity %s" % self.object.name


class EntityDeleteView(RelatedItemsMixin, RedirectToNextMixin, SetHeadlineMixin, MultiplePermissionsRequiredMixin, DeleteView):
    """
    View for deleting an entity.
    """

    model = Entity
    template_name = "conntrackt/delete_form.html"

    # Required permissions.
    permissions = {
        "all": ("conntrackt.delete_entity",),
        }

    # Raise authorisation denied exception for unmet permissions.
    raise_exception = True

    def post(self, *args, **kwargs):
        """
        Add a success message that will be displayed to the user to confirm the
        entity deletion.
        """

        messages.success(self.request, "Entity %s has been removed." % self.get_object().name, extra_tags="alert alert-success")

        return super(EntityDeleteView, self).post(*args, **kwargs)

    def delete(self, *args, **kwargs):
        """
        Deletes the object. This method is overridden in order to obtain the
        project ID for success URL.

        @TODO: Fix this once Django 1.6 comes out with fix from ticket 19044.
        """

        self.success_url = reverse("project", args=(self.get_object().project.id,))

        return super(EntityDeleteView, self).delete(*args, **kwargs)

    def get_headline(self):
        """
        Set headline based on entity name.
        """

        return "Delete entity %s" % self.object.name


class InterfaceCreateView(RedirectToNextMixin, SetHeadlineMixin, MultiplePermissionsRequiredMixin, CreateView):
    """
    View for creating a new interface.
    """

    model = Interface
    form_class = InterfaceForm
    headline = "Add new interface"
    template_name = "conntrackt/create_form.html"

    # Required permissions
    permissions = {
        "all": ("conntrackt.add_interface",),
        }

    # Raise authorisation denied exception for unmet permissions.
    raise_exception = True

    def get_form(self, form_class):
        """
        Returns an instance of form that can be used by the view.

        The method will limit the entity select input if request contained this
        information.
        """

        form = super(InterfaceCreateView, self).get_form(form_class)

        # Limit the entity selection if required.
        entity_id = self.request.GET.get("entity", None)
        if entity_id:
            form.fields["entity"].queryset = Entity.objects.filter(pk=entity_id)
            form.fields["entity"].empty_label = False

        return form

    def get_initial(self):
        """
        Returns initial values that should be pre-selected (if they were
        specified through a GET parameter).
        """

        initial = super(InterfaceCreateView, self).get_initial()

        initial["entity"] = self.request.GET.get("entity", None)

        return initial

    def get_success_url(self):
        """
        Returns the URL to which the user should be redirected after an
        interface has been created.

        The URL in this case will be set to entity's details page.
        """

        return reverse("entity", args=(self.object.entity.pk,))


class InterfaceUpdateView(RedirectToNextMixin, SetHeadlineMixin, MultiplePermissionsRequiredMixin, UpdateView):
    """
    View for updating an existing interface.
    """

    model = Interface
    form_class = InterfaceForm
    template_name = "conntrackt/update_form.html"

    # Required permissions.
    permissions = {
        "all": ("conntrackt.change_interface",),
        }

    # Raise authorisation denied exception for unmet permissions.
    raise_exception = True

    def get_form(self, form_class):
        """
        Returns an instance of form that can be used by the view.

        The method will limit the entities that can be selected for the
        interface to the ones that belong to the same project as the currently
        set entity.
        """

        form = super(InterfaceUpdateView, self).get_form(form_class)

        # Limit the entities to same project.
        form.fields["entity"].queryset = Entity.objects.filter(project=self.object.entity.project)

        return form

    def get_success_url(self):
        """
        Returns the URL to which the user should be redirected after an
        interface has been updated.

        The URL in this case will be set to entity's details page.
        """

        return reverse("entity", args=(self.object.entity.pk,))

    def get_headline(self):
        """
        Set headline based on interface name.
        """

        return "Update interface %s" % self.object.name


class InterfaceDeleteView(RelatedItemsMixin, RedirectToNextMixin, SetHeadlineMixin, MultiplePermissionsRequiredMixin, DeleteView):
    """
    View for deleting an interface.
    """

    model = Interface
    template_name = "conntrackt/delete_form.html"

    # Required permissions.
    permissions = {
        "all": ("conntrackt.delete_interface",),
        }

    # Raise authorisation denied exception for unmet permissions.
    raise_exception = True

    def post(self, *args, **kwargs):
        """
        Add a success message that will be displayed to the user to confirm the
        interface deletion.
        """

        messages.success(self.request, "Interface %s has been removed." % self.get_object().name, extra_tags="alert alert-success")

        return super(InterfaceDeleteView, self).post(*args, **kwargs)

    def delete(self, *args, **kwargs):
        """
        Deletes the object. This method is overridden in order to obtain the
        entity ID for success URL.

        @TODO: Fix this once Django 1.6 comes out with fix from ticket 19044.
        """

        self.success_url = reverse("entity", args=(self.get_object().entity.id,))

        return super(InterfaceDeleteView, self).delete(*args, **kwargs)

    def get_headline(self):
        """
        Set headline based on interface name.
        """

        return "Delete interface %s" % self.object.name


class CommunicationCreateView(RedirectToNextMixin, SetHeadlineMixin, MultiplePermissionsRequiredMixin, CreateView):
    """
    View for creating a new communication.
    """

    model = Communication
    form_class = CommunicationForm
    headline = "Add new communication"
    template_name = "conntrackt/create_form.html"

    # Required permissions
    permissions = {
        "all": ("conntrackt.add_communication",),
        }

    # Raise authorisation denied exception for unmet permissions.
    raise_exception = True

    def get_form(self, form_class):
        """
        Returns an instance of form that can be used by the view.

        The method will limit the source and destination interface selection to
        interfaces belonging to the same project as provided entity ID (if any
        was provided).
        """

        form = super(CommunicationCreateView, self).get_form(form_class)

        # Limit the interface selection based on provided source entity,
        # destination entity, or project.
        entity_id = self.request.GET.get("from_entity", None) or self.request.GET.get("to_entity", None)
        project_id = self.request.GET.get("project", None)

        if project_id:
            form.fields["source"].queryset = Interface.objects.filter(entity__project=project_id)
            form.fields["destination"].queryset = Interface.objects.filter(entity__project=project_id)
        elif entity_id:
            entity = Entity.objects.get(pk=1)
            form.fields["source"].queryset = Interface.objects.filter(entity__project=entity.project)
            form.fields["destination"].queryset = Interface.objects.filter(entity__project=entity.project)

        return form

    def get_initial(self):
        """
        Returns initial values that should be pre-selected (if they were
        specified through a GET parameter).
        """

        initial = super(CommunicationCreateView, self).get_initial()

        # If source or destination entity were specified in request, fetch the
        # first interface from them and use it as initial source and destination.
        from_entity = self.request.GET.get("from_entity", None)
        to_entity = self.request.GET.get("to_entity", None)

        if from_entity:
            try:
                interface = Interface.objects.filter(entity=from_entity)[0]
                initial["source"] = interface.id
            except IndexError:
                pass

        if to_entity:
            try:
                interface = Interface.objects.filter(entity=to_entity)[0]
                initial["destination"] = interface.id
            except IndexError:
                pass

        return initial

    def get_success_url(self):
        """
        Returns the URL to which the user should be redirected after a
        communication has been created.

        The URL will either point to value provided via GET parameter "next", or
        to project page to which the communication belongs.
        """

        # We must set the success URL to something first.
        self.success_url = reverse("project", args=(self.object.source.entity.project.pk,))

        # This will override the URL if parameter "next" was provided (from
        # RedirectToNextMixin).
        success_url = super(CommunicationCreateView, self).get_success_url()

        return success_url


class CommunicationUpdateView(RedirectToNextMixin, SetHeadlineMixin, MultiplePermissionsRequiredMixin, UpdateView):
    """
    View for updating an existing communication.
    """

    model = Communication
    form_class = CommunicationForm
    template_name = "conntrackt/update_form.html"

    # Required permissions.
    permissions = {
        "all": ("conntrackt.change_communication",),
        }

    # Raise authorisation denied exception for unmet permissions.
    raise_exception = True

    def get_form(self, form_class):
        """
        Returns an instance of form that can be used by the view.

        The method will limit the source and destination interfaces that can be
        selected for the communication. Both will be limited to interfaces
        coming from entities that belong to the same project as current
        communication's source interface.
        """

        form = super(CommunicationUpdateView, self).get_form(form_class)

        project = self.object.source.entity.project

        form.fields["source"].queryset = Interface.objects.filter(entity__project=project)
        form.fields["destination"].queryset = Interface.objects.filter(entity__project=project)

        return form

    def get_success_url(self):
        """
        Returns the URL to which the user should be redirected after a
        communication has been created.

        The URL will either point to value provided via GET parameter "next", or
        to project page to which the communication belongs.
        """

        return self.request.GET.get("next", reverse("project", args=(self.object.source.entity.project.pk,)))

    def get_headline(self):
        """
        Set headline based on communication.
        """

        return "Update communication %s" % self.object


class CommunicationDeleteView(RelatedItemsMixin, RedirectToNextMixin, SetHeadlineMixin, MultiplePermissionsRequiredMixin, DeleteView):
    """
    View for deleting an communication.
    """

    model = Communication
    template_name = "conntrackt/delete_form.html"

    # Required permissions.
    permissions = {
        "all": ("conntrackt.delete_communication",),
        }

    # Raise authorisation denied exception for unmet permissions.
    raise_exception = True

    def post(self, *args, **kwargs):
        """
        Add a success message that will be displayed to the user to confirm the
        communication deletion.
        """

        messages.success(self.request, "Communication %s has been removed." % self.get_object(), extra_tags="alert alert-success")

        return super(CommunicationDeleteView, self).post(*args, **kwargs)

    def delete(self, *args, **kwargs):
        """
        Deletes the object. This method is overridden in order to obtain the
        entity ID for success URL.

        @TODO: Fix this once Django 1.6 comes out with fix from ticket 19044.
        """

        entity_id = self.request.GET.get("from_entity", None) or self.request.GET.get("to_entity", None) or self.get_object().source.entity.pk

        self.success_url = reverse("entity", args=(entity_id,))

        return super(CommunicationDeleteView, self).delete(*args, **kwargs)

    def get_headline(self):
        """
        Set headline based on communication.
        """

        return "Delete communication %s" % self.object


@permission_required("conntrackt.view", raise_exception=True)
def project_diagram(request, pk):
    """
    Custom view that returns response containing diagram of project
    communications.

    The diagram will include coloured entities, with directional lines
    connecting the source and destination end entities.

    The output format is SVG.

    Arguments:

        request - Request object.

        pk - Project ID for which the diagram should be generated.

    Returns:

        Response object that contains the project diagram rendered as SVG.
    """

    # Fetch the project.
    project = get_object_or_404(Project, pk=pk)

    # Generate the diagram.
    content = generate_project_diagram(project).create_svg()

    # Set the mime type.
    response = HttpResponse(content, mimetype='image/svg+xml')

    # Return the response object.
    return response


class SearchView(MultiplePermissionsRequiredMixin, TemplateView):
    """
    Custom view used for rendering the search (results) page.
    """

    template_name = 'conntrackt/search.html'

    # Required permissions.
    permissions = {
        "all": ("conntrackt.view",),
        }

    # Raise authorisation denied exception for unmet permissions.
    raise_exception = True

    def get_context_data(self, **kwargs):
        """
        Returns the context data that should be used for rendering of the
        template.

        Adds context objects:
          - 'entities', which is a list of entities that had the search term in
            their name or description.
          - 'projects', which is a list of entities that had the search term in
            their name or description.
          - 'search_term', which is a string of previous query that brought the
            user to page (if any). The term will be stripped from leading and
            trailing spaces/tabs.
        """

        # Set the context using the parent aclass.
        context = super(SearchView, self).get_context_data(**kwargs)

        # Retrieve the search term, and strip it if it was provided.
        search_term = self.request.GET.get("q", None)
        if search_term:
            search_term = search_term.strip()

        # Do not allow empty searches.
        if search_term == "":
            messages.error(self.request, "Search query is not allowed to be empty.", extra_tags="alert alert-error")
        # Set-up the context objects if search was sent. Otherwise empty search
        # page will be shown.
        elif search_term is not None:
            context['search_term'] = search_term
            context['entities'] = Entity.objects.search(search_term)
            context['projects'] = Project.objects.search(search_term)

        return context


class APISearchView(MultiplePermissionsRequiredMixin, View):
    """
    API view implementing search for entities and projects that match the
    provided search term.

    The output generated by the view uses JSON. The result will include a list
    of matched items, where each item is a dictionary with the following keys:

      - name (name of the matched item)
      - project (project to which the item belongs)
      - type (type of the matched item)
      - url (URL towards the matched item)
    """

    # Required permissions.
    permissions = {
        "all": ("conntrackt.view",),
        }

    # Raise authorisation denied exception for unmet permissions.
    raise_exception = True

    def get(self, request, search_term=""):
        """
        Implements response handling for a GET request.
        """

        # Retrieve the search term, and strip it if it was provided.
        if search_term:
            search_term = search_term.strip()

        # Set-up a list that will contain found items.
        items = []

        # Fetch the maximum number of items that should be returned.
        limit = int(request.GET.get("limit", 0))
        if limit < 0:
            raise ValidationError("Limit may not be a negative value.")

        # Don't perform search with empty search term.
        if search_term != "":

            # Run the search on entities and projects.
            entities = Entity.objects.search(search_term).select_related("project")
            projects = Project.objects.search(search_term)

            # If maximum number of items was provided, narrow-down the results.
            if limit > 0:
                entities = entities[:limit]
                projects = projects[:limit]

            # Add found entities.
            for entity in entities:
                items.append({"name": entity.name,
                              "project": entity.project.name,
                              "type": "entity",
                              "url": entity.get_absolute_url()})

            # Add found projects.
            for project in projects:
                items.append({"name": project.name,
                              "project": project.name,
                              "type": "project",
                              "url": project.get_absolute_url()})

        # Generate the JSON response.
        content = json.dumps(items)
        response = HttpResponse(content, mimetype="application/json")

        # Return the response.
        return response



# -*- coding: utf-8 -*-
#
# Copyright (C) 2013 Branko Majic
#
# This file is part of Django Conntrackt.
#
# Django Conntrackt is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# Django Conntrackt is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# Django Conntrackt.  If not, see <http://www.gnu.org/licenses/>.
#

# Standard Python library import.
import collections

# Python third-party library imports.
import mock
from palette import Color

# Django imports.
from django.core.exceptions import ValidationError
from django.db import IntegrityError
from django.db.models import Model
from django.test import TestCase

# Application imports.
from conntrackt.models import Project, Location, Entity, Interface, Communication
from conntrackt.models import SearchManager
from conntrackt.models import NestedObjects
from conntrackt.utils import list_formatter_callback

# Test imports.
from .factories import ProjectFactory, LocationFactory
from .factories import ServerEntityFactory, ServerInterfaceFactory
from .factories import SubnetEntityFactory, SubnetInterfaceFactory
from .factories import CommunicationFactory
from .factories import setup_test_data


class RelatedCollectorMixinTest(TestCase):

    @mock.patch.object(NestedObjects, "collect")
    @mock.patch.object(NestedObjects, "nested")
    def test_get_dependant_objects_method_calls(self, nested_mock, collect_mock):
        """
        Tests if correct methods are being called with correct arguments during
        the invocation of get_dependant_objects method.
        """

        # Set-up some test data.
        project = ProjectFactory()

        # Call the method.
        project.get_dependant_objects()

        # Check if correct collector methods were called.
        collect_mock.assert_called_with([project])
        nested_mock.assert_called_with()

    def test_get_dependant_objects_return_value(self):
        """
        Tests the return value of get_dependant_objects method.
        """

        # Set-up some test data.
        project = ProjectFactory()
        location = LocationFactory()
        entity1 = ServerEntityFactory(pk=1, project=project, location=location)
        entity2 = ServerEntityFactory(pk=2, project=project, location=location)
        communication1 = CommunicationFactory(pk=1, source_id=1, destination_id=2, protocol="TCP", port="22")

        # Get the dependant objects.
        dependant_objects = project.get_dependant_objects()

        # Create a small local function for traversing the recursive list.
        def traverse(data):
            # If data is iterable, verify it is a list, and process its members
            # as well. If data is not iterable, make sure it is descendant of
            # Django Model class.
            if isinstance(data, collections.Iterable):
                self.assertIsInstance(data, list)
                for element in data:
                    traverse(element)
            else:
                self.assertIsInstance(data, Model)

        # Traverse the obtained dependant objects.
        traverse(dependant_objects)

    @mock.patch.object(NestedObjects, "collect")
    @mock.patch.object(NestedObjects, "nested")
    def test_get_dependant_objects_representation_method_calls(self, nested_mock, collect_mock):
        """
        Tests if correct methods are being called with correct arguments during
        the invocation of get_dependant_objects method.
        """

        # Set-up some test data.
        project = ProjectFactory()

        # Call the method.
        project.get_dependant_objects_representation()

        # Check if correct collector methods were called.
        collect_mock.assert_called_with([project])
        nested_mock.assert_called_with(list_formatter_callback)

    def test_get_dependant_objects_representation_return_value(self):
        """
        Tests the return value of get_dependant_objects_representation method.
        """

        # Set-up some test data.
        project = ProjectFactory()
        location = LocationFactory()
        entity1 = ServerEntityFactory(pk=1, project=project, location=location)
        entity2 = ServerEntityFactory(pk=2, project=project, location=location)
        communication1 = CommunicationFactory(pk=1, source_id=1, destination_id=2, protocol="TCP", port="22")

        # Get the dependant objects.
        dependant_objects = project.get_dependant_objects_representation()

        # Create a small local function for traversing the recursive list.
        def traverse(data):
            # If data is iterable, verify it is a list, and process its members
            # as well. If data is not iterable, make sure it is descendant of
            # Django Model class.
            if isinstance(data, collections.Iterable) and not isinstance(data, str):
                self.assertIsInstance(data, list)
                for element in data:
                    traverse(element)
            else:
                self.assertIsInstance(data, str)

        # Traverse the obtained dependant objects.
        traverse(dependant_objects)


class ProjectTest(TestCase):

    def test_unique_name(self):
        """
        Test if unique project name is enforced.
        """

        project = ProjectFactory()

        self.assertRaises(IntegrityError, ProjectFactory, name=project.name)

    def test_representation(self):
        """
        Test the representation of project.
        """

        project = ProjectFactory(name="Test Project")

        self.assertEqual(str(project), "Test Project")

    def test_absolute_url(self):
        """
        Tests if the absolute URL is generated properly.
        """

        project = ProjectFactory(pk=1)

        self.assertEqual(project.get_absolute_url(), "/conntrackt/project/1/")

    def test_custom_manager(self):
        """
        Tests if the custom manager is being used.
        """

        self.assertIsInstance(Project.objects, SearchManager)

    def test_get_project_communications_summary_count(self):
        """
        Test if the method returns correct number of entries in the list.
        """

        # Set-up some test data to work with.
        setup_test_data()

        # Get the first project from test data.
        project = Project.objects.get(pk=1)

        # Fetch the project communications
        communications = project.get_project_communications_summary()

        # Validate the number of returned communications.
        self.assertEqual(len(communications), 6)

    def test_get_project_communications_summary_return_value(self):
        """
        Test if the method returns correct type.
        """

        # Set-up some test data.
        setup_test_data()

        # Fetch one of the projects.
        project = Project.objects.get(pk=1)

        # Get the communications summary for the project.
        communications = project.get_project_communications_summary()

        # Validate the return value type.
        self.assertIsInstance(communications, list)

        # Perform verification on every summary element returned.
        for comm in communications:
            # Verify the type of element and its size.
            self.assertIsInstance(comm, dict)
            self.assertEqual(len(comm), 6)

            # Verify the presence of correct dictionary keys.
            keys = comm.keys()
            self.assertIn("source", keys)
            self.assertIn("source_color", keys)
            self.assertIn("destination", keys)
            self.assertIn("destination_color", keys)
            self.assertIn("protocol", keys)
            self.assertIn("port", keys)

            # Verify the value types.
            self.assertIsInstance(comm["source"], unicode)
            self.assertIsInstance(comm["source_color"], Color)
            self.assertIsInstance(comm["destination"], unicode)
            self.assertIsInstance(comm["destination_color"], Color)
            self.assertIsInstance(comm["protocol"], unicode)
            self.assertIsInstance(comm["port"], int)


class LocationTest(TestCase):

    def test_unique_name(self):
        """
        Test if unique location name is enforced.
        """

        location = LocationFactory()

        self.assertRaises(IntegrityError, LocationFactory, name=location.name)

    def test_representation(self):
        """
        Test the representation of location.
        """

        project = LocationFactory(name="Test Location")

        self.assertEqual(str(project), "Test Location")


class EntityTest(TestCase):

    def setUp(cls):
        """
        Set-up some test data.
        """

        setup_test_data()

    def test_incoming_communications(self):
        """
        Test that we get correct list of incoming connections with the sample
        data.
        """

        entity = Entity.objects.get(pk=1)
        incoming = Communication.objects.filter(pk__in=(1, 2, 3, 5))

        self.assertItemsEqual(entity.incoming_communications(), incoming)

    def test_outgoing_communications(self):
        """
        Test that we get correct list of outgoing connections with the sample
        data.
        """

        entity = Entity.objects.get(pk=1)
        outgoing = Communication.objects.filter(pk__in=(4, 6))

        self.assertItemsEqual(entity.outgoing_communications(), outgoing)

    def test_representation(self):
        """
        Test the representation of entity.
        """

        ent = Entity.objects.get(pk=1)
        representation = "Test Entity 1 (Test Project 1 - Test Location 1)"

        self.assertEqual(str(ent), representation)

    def test_unique_name(self):
        """
        Test if unique entity name is enforced across same project.
        """

        entity1 = Entity.objects.get(pk=1)

        entity_dup = Entity(name=entity1.name, description="Duplicate entity.", project=entity1.project, location=entity1.location)

        self.assertRaises(IntegrityError, entity_dup.save)

    def test_absolute_url(self):
        """
        Tests if the absolute URL is generated properly.
        """

        entity = Entity.objects.get(pk=1)

        self.assertEqual(entity.get_absolute_url(), "/conntrackt/entity/1/")

    def test_project_move_constraints(self):
        """
        Tests if entity is prevented from being moved to different project in
        case of existing communications.
        """

        entity = Entity.objects.get(pk=1)
        new_project = Project.objects.get(pk=2)

        entity.project = new_project
        self.assertRaisesRegexp(ValidationError, "The entity cannot be moved to different project as long as it has valid communications with entities in current project.", entity.clean)

    def test_custom_manager(self):
        """
        Tests if the custom manager is being used.
        """

        self.assertIsInstance(Entity.objects, SearchManager)


class InterfaceTest(TestCase):

    def setUp(self):
        """
        Set-up some test data.
        """

        setup_test_data()

    def test_unique_name(self):
        """
        Test if unique interface name is enforced across same entity.
        """

        entity = Entity.objects.get(pk=1)

        interface = entity.interface_set.get(pk=1)

        duplicate = Interface(name=interface.name, description="Duplicate interface.", entity=entity, address="10.10.10.10", netmask="255.255.255.255")

        self.assertRaises(IntegrityError, duplicate.save)

    def test_unique_address(self):
        """
        Test if unique address/netmask is enforced across same entity.
        """

        entity = Entity.objects.get(pk=1)

        interface = entity.interface_set.get(pk=1)

        duplicate = Interface(name="eth1", description="Duplicate address", entity=entity, address=interface.address, netmask=interface.netmask)

        self.assertRaises(IntegrityError, duplicate.save)

    def test_representation_single(self):
        """
        Test representation of single IP address.
        """

        interface = Entity.objects.get(name="Test Entity 1").interface_set.get(name="eth0")
        representation = "Test Entity 1 (192.168.1.1)"

        self.assertEqual(str(interface), representation)

    def test_representation_subnet(self):
        """
        Test representation of subnet.
        """

        interface = Entity.objects.get(pk=4).interface_set.get(name="switch0")
        representation = "Test Subnet 4 (10.10.4.0/255.255.255.0)"

        self.assertEqual(str(interface), representation)


class CommunicationTest(TestCase):

    def setUp(self):
        """
        Set-up some test data.
        """

        setup_test_data()

    def test_unique_communication(self):
        """
        Test enforcement of unique communications.
        """

        comm = Communication.objects.get(pk=1)

        self.assertRaises(IntegrityError, Communication.objects.create, source=comm.source, destination=comm.destination, protocol=comm.protocol, port=comm.port, description="Duplicate communication.")

    def test_project_same(self):
        """
        Test enforcement of same project entities for communications.
        """

        ent1 = Entity.objects.get(pk=1)
        ent1_eth0 = ent1.interface_set.get(name="eth0")
        ent5 = Entity.objects.get(pk=5)
        ent5_eth0 = ent5.interface_set.get(name="eth0")

        # Set-up a communication between different projects.
        comm = Communication.objects.create(source=ent1_eth0, destination=ent5_eth0, protocol="ICMP", port="8", description="Ping.")

        self.assertRaisesRegexp(ValidationError, 'Source and destination entities do not belong to the same project', comm.full_clean)

    def test_same_entity(self):
        """
        Test enforcement of differing entities for communication.
        """

        ent = Entity.objects.get(pk=1)
        ent_eth0 = ent.interface_set.get(name="eth0")

        # Set-up a communication between same entity.
        comm = Communication.objects.create(source=ent_eth0, destination=ent_eth0, protocol="ICMP", port="8", description="Ping.")

        self.assertRaisesRegexp(ValidationError, "Source and destination entities are identical.", comm.full_clean)

    def test_unsupported_protocol(self):
        """
        Test enforcement of supported protocol.
        """

        ent1 = Entity.objects.get(pk=1)
        ent1_eth0 = ent1.interface_set.get(name="eth0")
        ent2 = Entity.objects.get(pk=2)
        ent2_eth0 = ent2.interface_set.get(name="eth0")

        comm = Communication(source=ent1_eth0, destination=ent2_eth0, protocol="BOGUS", port="1234")

        self.assertRaisesRegexp(ValidationError, "BOGUS is not a supported protocol.", comm.full_clean)

    def test_edit_link(self):
        """
        Tests the function for getting the edit link string.
        """

        comm = Communication.objects.get(pk=1)

        self.assertEqual("Edit", comm.edit_link())

    def test_representation(self):
        """
        Test the representation of communication.
        """

        comm = Communication.objects.get(pk=1)

        expected = "Test Entity 2 -> Test Entity 1 (TCP:22)"

        self.assertEqual(expected, str(comm))

    def test_source_representation(self):
        """
        Test the representation of communication from source perspective.
        """

        comm = Communication.objects.get(pk=1)

        expected = "Test Entity 2 (192.168.1.2) - TCP: 22"

        self.assertEqual(expected, comm.source_representation())

    def test_destination_representation(self):
        """
        Test the representation of communication from destination perspective.
        """

        comm = Communication.objects.get(pk=1)

        expected = "Test Entity 1 (192.168.1.1) - TCP: 22"

        self.assertEqual(expected, comm.destination_representation())



# -*- coding: utf-8 -*-
#
# Copyright (C) 2013 Branko Majic
#
# This file is part of Django Conntrackt.
#
# Django Conntrackt is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# Django Conntrackt is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# Django Conntrackt.  If not, see <http://www.gnu.org/licenses/>.
#


# -*- coding: utf-8 -*-
#
# Copyright (C) 2013 Branko Majic
#
# This file is part of Django Conntrackt.
#
# Django Conntrackt is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# Django Conntrackt is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# Django Conntrackt.  If not, see <http://www.gnu.org/licenses/>.
#


# Django imports.
from django.contrib import admin
from django.core.urlresolvers import resolve

# Application imports.
from .models import Project, Location, Entity, Interface, Communication


class InterfaceInline(admin.StackedInline):
    """
    This class implements the inline admin view of the Interface instances. This
    is used when adding the entities (since it's easier to specify interface for
    an entity right away).

    Properties:

      - model - Model that this admin class refers to.
      - extra - Number of interfaces that should be show inline for
        adding/editing.
    """

    model = Interface
    extra = 1


class CommunicationProjectListFilter(admin.SimpleListFilter):
    """
    This class implements a project-based filter for the communications list
    view. The filter is applied on both the source and destination field of a
    communication.

    The filter assumes that the communication belongs to a project by following
    the relationships through source and destination field towards interface,
    then entity, and then finally entity's project.

    Both source and destination must fullfil the requirement of belonging to the
    same project in order for the communication to be part of the resulting
    queryset.
    """

    # Set-up the filter title and parameter name that will be used for GET
    # request.
    title = "project"
    parameter_name = "project"

    def lookups(self, request, model_admin):
        """
        Returns a list of tuples that provide possible filter values that can be
        applied.

        Arguments:

          request - Request associated with the calling view.

          model_admin - Modem admin that can be used for accessing the model
          data.

        Returns:

          List of (project_id, project_object) tuples.
        """

        return [(p.id, p) for p in Project.objects.all()]

    def queryset(self, request, queryset):
        """
        Applies filtering by project ID on the provided communication queryset.

        Arguments:

          request - Request associated with the calling view.

          queryset - Current queryset used for displaying the information in the
          view.

        Returns:

          Queryset with applied filtering by object (if any). If no filtering
          needs to be done, returns original queryset.
        """

        # Apply the project filter on source and destination entity's project
        # ID, if it was specified.
        if self.value():
            return queryset.filter(source__entity__project=self.value(),
                                   destination__entity__project=self.value())

        return queryset


class CommunicationLocationListFilter(admin.SimpleListFilter):
    """
    This class implements a location-based filter for the communications list
    view. The filter is applied on both the source and destination field of a
    communication.

    The filter assumes that the communication belongs to a location by following
    the relationships through source and destination field towards interface,
    then entity, and then finally entity's location.

    Both source and destination must fullfil the requirement of belonging to the
    same location in order for the communication to be part of the resulting
    queryset.
    """

    # Set-up the filter title and parameter name that will be used for GET
    # request.
    title = "location"
    parameter_name = "location"

    def lookups(self, request, model_admin):
        """
        Returns a list of tuples that provide possible filter values that can be
        applied.

        Arguments:

          request - Request associated with the calling view.

          model_admin - Modem admin that can be used for accessing the model
          data.

        Returns:

          List of (project_id, project_object) tuples.
        """

        return [(p.id, p) for p in Location.objects.all()]

    def queryset(self, request, queryset):
        """
        Applies filtering by project ID on the provided communication queryset.

        Arguments:

          request - Request associated with the calling view.

          queryset - Current queryset used for displaying the information in the
          view.

        Returns:

          Queryset with applied filtering by object (if any). If no filtering
          needs to be done, returns original queryset.
        """

        # Apply the location filter on source and destination entity's project
        # ID, if it was specified.
        if self.value():
            return queryset.filter(source__entity__location=self.value(),
                                   destination__entity__location=self.value())

        return queryset


class CommunicationAdmin(admin.ModelAdmin):
    """
    Modifies the default admin class for the Communication class. The
    communication class needs to be modified in a number of ways in order to
    cater for easier adding of communication links, letting us limit the
    interfaces being shown as source/destination to specific project and/or
    site.
    """

    # Show standard fields of the model, and also include a separate edit link
    # so that other fields can be editable.
    list_display = ('source', 'destination', 'protocol', 'port', 'edit_link')
    # Make the extra edit link the main link for bringing-up admin page for
    # editing the communication.
    list_display_links = ('edit_link',)
    # All of the fields should be editable inline for ease-of-use purposes.
    list_editable = ('source', 'destination', 'protocol', 'port')
    # Add filters for project/location.
    list_filter = (CommunicationProjectListFilter, CommunicationLocationListFilter)

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        """
        Overrides the default queryset for the foreign key fields. This lets us
        limit the specification of communication to specific project and/or
        location. These are in turn passed through the GET parameters.

        Arguments:

          db_field - Name of the model field for which this method is called.

          request - Request associated with the calling view.

          kwargs - Additional keyword arguments
        """

        # Resolve the view name based on the request's path.
        view_name = resolve(request.path).view_name

        # Only process the source and destination fields that point to
        # interfaces.
        if db_field.name == "source" or db_field.name == "destination":
            # Perform no filtering by default.
            interface_filter = {}

            # If project was specified in GET requests, add it as a filter.
            if 'project' in request.GET:
                interface_filter['entity__project'] = request.GET['project']
            # If location was specified in GET request, add it as a filter.
            if 'location' in request.GET:
                interface_filter['entity__location'] = request.GET['location']
            # If there are any filtering options for the show interfaces, apply them.
            if interface_filter:
                kwargs["queryset"] = Interface.objects.filter(**interface_filter)

        # Call the parent's method so it would do any of its magic.
        return super(CommunicationAdmin, self).formfield_for_foreignkey(db_field, request, **kwargs)


class EntityAdmin(admin.ModelAdmin):
    """
    This class implements the admin view of the entity instances. It adds some
    inline capability that can be edited for the entity, and also adds inline
    editing of interfaces related to the entity.
    """

    # Show the interfaces inline when editing an entity.
    inlines = [InterfaceInline]
    # Specify what should be viewed in a list display.
    list_display = ('name', 'project', 'location')
    # Allow the user to change project and location directly in the list.
    list_editable = ('project', 'location')
    # Enable filtering based on project and location.
    list_filter = ['project', 'location']


class InterfaceAdmin(admin.ModelAdmin):
    """
    This class implements the admin view of the interface instances. It allows
    editing the IP address and netmask of an interface directly in the listing.

    It also adds some filtering capability based on project and/or location.
    """

    # Specify fields that should be visible in the list view.
    list_display = ('entity', 'address', 'netmask')
    # Allow changing of IP address and netmask directly in the list view.
    list_editable = ('address', 'netmask')
    # Enable filtering based on project and location.
    list_filter = ['entity__project', 'entity__location']


# Register our admin classes.
admin.site.register(Project)
admin.site.register(Location)
admin.site.register(Entity, EntityAdmin)
admin.site.register(Interface, InterfaceAdmin)
admin.site.register(Communication, CommunicationAdmin)
